package Drawings;
import Entities.EntityTrain;

import java.awt.*;
import java.util.Random;

public class DrawingTrain {
    private EntityTrain entityTrain;
    public EntityTrain getEntityTrain(){return entityTrain;}
    private IDrawingWheels drawingWheels;

    private Integer _pictureWidth;
    private Integer _pictureHeight;
    protected Integer _startPosX;
    protected Integer _startPosY;
    private int _drawingTrainWidth = 100;
    private int _drawingTrainHeight = 40;
    public int getPosX(){return _startPosX;}
    public int getPosY(){return _startPosY;}
    public int getTrainWidth(){return _drawingTrainWidth;}
    public int getTrainHeight(){return _drawingTrainHeight;}

    public DrawingTrain(int speed, double weight, Color bodyColor, int wheelsType){
        entityTrain = new EntityTrain(speed, weight, bodyColor);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
        switch(wheelsType){
            case 0:
                drawingWheels = new DrawingMonorailWheels();
                break;
            case 1:
                drawingWheels = new DrawingMonorailWheelsPlus();
                break;
            case 2:
                drawingWheels = new DrawingMonorailWheelsSquare();
                break;
            default:
                break;
        }
        Random rand = new Random();
        int wheelsCount = rand.nextInt(2, 5);
        drawingWheels.setEnumNumber(wheelsCount);
    }

    public DrawingTrain(int speed, double weight, Color bodyColor, int wheelsType, int drawingTrainWidth, int drawingTrainHeight){
        this(speed, weight, bodyColor, wheelsType);
        _drawingTrainWidth = drawingTrainWidth;
        _drawingTrainHeight = drawingTrainHeight;
    }

    public boolean SetPictureSize(int width, int height){
        if (_drawingTrainWidth <= width && _drawingTrainHeight <= height){
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX != null && _startPosY != null){
                if (_startPosX + _drawingTrainWidth > _pictureWidth) {
                    _startPosX = _pictureWidth - _drawingTrainWidth;
                }
                if (_startPosY + _drawingTrainHeight > _pictureHeight) {
                    _startPosY = _pictureHeight - _drawingTrainHeight;
                }
            }
            return true;
        }
        return false;
    }

    public void SetPosition(int x, int y)
    {
        if (_pictureHeight == null || _pictureWidth == null) {
            return;
        }
        if (x + _drawingTrainWidth > _pictureWidth || x < 0)
        {
            x = 0;
        }
        if (y + _drawingTrainHeight > _pictureHeight || y < 0)
        {
            y = 0;
        }
        _startPosX = x;
        _startPosY = y;
    }

    public boolean MoveTransport(DirectionType direction){
        if (entityTrain == null || _startPosX == null || _startPosY == null){
            return false;
        }

        switch (direction){
            case DirectionType.Up:
                if (_startPosY - entityTrain.Step() > 0){
                    _startPosY -= (int) entityTrain.Step();
                }
                return true;
            case DirectionType.Down:
                if (_startPosY + entityTrain.Step() + _drawingTrainHeight < _pictureHeight){
                    _startPosY += (int) entityTrain.Step();
                }
                return true;
            case DirectionType.Left:
                if (_startPosX - entityTrain.Step() > 0){
                    _startPosX -= (int) entityTrain.Step();
                }
                return true;
            case DirectionType.Right:
                if (_startPosX + entityTrain.Step() + _drawingTrainWidth < _pictureWidth){
                    _startPosX += (int) entityTrain.Step();
                }
                return true;
            default: return false;
        }
    }

    public void DrawTrain(Graphics g) {
        if (entityTrain == null || _startPosX == null || _startPosY == null) {
            return;
        }

        drawingWheels.drawMonorailWheels(g, entityTrain.getBodyColor(), _startPosX, _startPosY);

        Graphics2D g2d = (Graphics2D) g;

        Point[] monorailBorders = new Point[] {
                new Point(_startPosX + 10, _startPosY),
                new Point(_startPosX + 85, _startPosY),
                new Point(_startPosX + 85, _startPosY + 30),
                new Point(_startPosX + 5, _startPosY + 30),
                new Point(_startPosX + 5, _startPosY + 15)
        };

        Polygon monorailPolygon = new Polygon();
        for (Point point : monorailBorders)
            monorailPolygon.addPoint(point.x, point.y);
        g2d.setColor(Color.BLACK);
        g2d.draw(monorailPolygon);
        g2d.setColor(entityTrain.getBodyColor());
        g2d.fill(monorailPolygon);

        Point[][] carts = new Point[][] {
                {new Point(_startPosX, _startPosY + 35),
                        new Point(_startPosX + 15, _startPosY + 35),
                        new Point(_startPosX + 15, _startPosY + 30),
                        new Point(_startPosX + 5, _startPosY + 30)},
                {new Point(_startPosX + 20, _startPosY + 30),
                        new Point(_startPosX + 25, _startPosY + 35),
                        new Point(_startPosX + 30, _startPosY + 35),
                        new Point(_startPosX + 35, _startPosY + 30)},
                {new Point(_startPosX + 55, _startPosY + 30),
                        new Point(_startPosX + 60, _startPosY + 35),
                        new Point(_startPosX + 65, _startPosY + 35),
                        new Point(_startPosX + 70, _startPosY + 30)},
                {new Point(_startPosX + 70, _startPosY + 30),
                        new Point(_startPosX + 75, _startPosY + 35),
                        new Point(_startPosX + 90, _startPosY + 35),
                        new Point(_startPosX + 85, _startPosY + 30)}
        };

        g2d.setColor(Color.BLACK);
        for (int i = 0; i < 4; i++){
            Polygon cartPolygon = new Polygon();
            for (Point point : carts[i])
                cartPolygon.addPoint(point.x, point.y);
            g2d.fill(cartPolygon);
        }

        g2d.drawRect(_startPosX + 17, _startPosY + 5, 17, 7);
        g2d.drawRect(_startPosX + 55, _startPosY + 5, 27, 7);
        g2d.drawRect(_startPosX + 40, _startPosY + 7, 10, 21);

        g2d.setColor(Color.CYAN);
        g2d.fillRect(_startPosX + 18, _startPosY + 6, 16, 6);
        g2d.fillRect(_startPosX + 56, _startPosY + 6, 26, 6);
        g2d.setColor(Color.WHITE);
        g2d.fillRect(_startPosX + 41, _startPosY + 8, 9, 20);
    }
}
