import Drawings.*;
import MovementStrategy.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.Random;
import java.util.List;

public class FormMonorail extends JFrame {
    protected DrawingTrain _drawingTrain;
    private AbstractStrategy _strategy;
    JPanel PictureBox;
    private JButton buttonCreateMonorail;
    private JButton buttonCreateTrain;
    private JButton buttonRight;
    private JButton buttonDown;
    private JButton buttonLeft;
    private JButton buttonUp;
    private JComboBox comboBoxStrategy;
    private JButton buttonStrategyStep;

    private List<JComponent> controls;

    public void createObject(String obj){
        Random rand = new Random();
        switch(obj){
            case "Train":
                _drawingTrain = new DrawingTrain(rand.nextInt(100, 300), rand.nextDouble(1000, 3000),
                        new Color(rand.nextInt(255), rand.nextInt(255), rand.nextInt(255)), rand.nextInt(0, 3));
                break;
            case "Monorail":
                _drawingTrain = new DrawingMonorail(rand.nextInt(100, 300), rand.nextDouble(1000, 3000),
                        new Color(rand.nextInt(255), rand.nextInt(255), rand.nextInt(255)), rand.nextInt(0, 3),
                        new Color(rand.nextInt(255), rand.nextInt(255), rand.nextInt(255)),
                        rand.nextBoolean(), rand.nextBoolean());
                break;
            default:
                return;
        }
        _drawingTrain.SetPictureSize(PictureBox.getWidth(), PictureBox.getHeight());
        _drawingTrain.SetPosition(rand.nextInt(50), rand.nextInt(50));
        _strategy = null;
        comboBoxStrategy.setEnabled(true);
        Draw();
    }

    public FormMonorail() {
        buttonUp.setName("buttonUp");
        buttonDown.setName("buttonDown");
        buttonLeft.setName("buttonLeft");
        buttonRight.setName("buttonRight");

        InitializeControlsRepaintList();

        buttonCreateMonorail.addActionListener(e -> createObject("Monorail"));

        buttonCreateTrain.addActionListener(e -> createObject("Train"));
        ActionListener buttonMoveClickedListener = e -> {
            String buttonName = ((JButton) e.getSource()).getName();
            boolean result = false;

            switch (buttonName) {
                case "buttonUp": {
                    result = _drawingTrain.MoveTransport(DirectionType.Up);
                }
                break;
                case "buttonDown": {
                    result = _drawingTrain.MoveTransport(DirectionType.Down);
                }
                break;
                case "buttonLeft": {
                    result = _drawingTrain.MoveTransport(DirectionType.Left);
                }
                break;
                case "buttonRight": {
                    result = _drawingTrain.MoveTransport(DirectionType.Right);
                }
                break;

            }
            if (result)
                Draw();

        };
        buttonRight.addActionListener(buttonMoveClickedListener);
        buttonDown.addActionListener(buttonMoveClickedListener);
        buttonLeft.addActionListener(buttonMoveClickedListener);
        buttonUp.addActionListener(buttonMoveClickedListener);

        comboBoxStrategy.addItem("К Центру");
        comboBoxStrategy.addItem("К Краю");
        buttonStrategyStep.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_drawingTrain == null){return;}
                if (comboBoxStrategy.isEnabled()){
                    switch (comboBoxStrategy.getSelectedIndex()){
                        case 0:
                            _strategy = new MoveToCenter();
                            break;
                        case 1:
                            _strategy = new MoveToBorder();
                            break;
                        default:
                            _strategy = null;
                            break;
                    }
                    if (_strategy == null) {return;}
                    _strategy.SetData(new MovableTrain(_drawingTrain), PictureBox.getWidth(), PictureBox.getHeight());
                }
                if (_strategy == null) {return;}
                _strategy.MakeStep();
                Draw();
                comboBoxStrategy.setEnabled(false);
                if (_strategy.GetStatus() == StrategyStatus.Finish) {
                    comboBoxStrategy.setEnabled(true);
                    _strategy = null;
                }
            }
        });
    }

    private void Draw() {
        if (_drawingTrain.getEntityTrain() == null)
            return;
        if (PictureBox.getWidth() == 0 || PictureBox.getHeight() == 0) {
            return;
        }
        Graphics g = PictureBox.getGraphics();
        g.setColor(PictureBox.getBackground());
        g.fillRect(0, 0, PictureBox.getWidth(), PictureBox.getHeight());
        _drawingTrain.DrawTrain(g);

        RepaintControls();

    }

    private void RepaintControls() {
        for (JComponent control : controls) {
            control.repaint();
        }
    }


    private void InitializeControlsRepaintList() {
        controls = new LinkedList<>();
        controls.add(buttonCreateMonorail);
        controls.add(buttonCreateTrain);
        controls.add(buttonUp);
        controls.add(buttonDown);
        controls.add(buttonLeft);
        controls.add(buttonRight);
    }
}
